--[[
-- For global instance
--]]

---@class neopyter.JupyterModule
---@field jupyterlab neopyter.JupyterLab|nil
---@field notebook neopyter.Notebook|nil
local M = {
    jupyterlab = nil,
    notebook = nil,
}

return M
