import { LabIcon } from '@jupyterlab/ui-components';
import statusStr from '../style/icons/statuspage.svg';

export const statusPageIcon = new LabIcon({
  name: 'neopyter:status-page',
  svgstr: statusStr
});
