"use strict";(self.webpackChunkneopyter=self.webpackChunkneopyter||[]).push([[186],{9186:e=>{e.exports=JSON.parse('{"$schema":"http://json-schema.org/draft-07/schema","$comment":"This schema is work with jupyter lab extension settings","jupyter.lab.setting-icon":"ui-components:neopyter","jupyter.lab.setting-icon-label":"Neopyter","definitions":{"ip":{"title":"IP","type":"string"},"port":{"title":"Port","type":"number"}},"properties":{"mode":{"title":"Work mode","type":"string","default":"direct","enum":["direct","proxy"]},"loglevel":{"title":"Log level","type":"string","default":"error","enum":["info","warn","debug","error"]}},"dependencies":{"mode":{"oneOf":[{"properties":{"mode":{"description":"In this mode, neovim is server and neovim plugin(neopyter) is listening to `remote_address`, the browser where jupyter lab is located will connect to neovim","enum":["direct"]},"ip":{"description":"IP of the host where neovim is located, neovim plugin(neopyter) will listen to the IP. e.g. `10.25.69.51`","$ref":"#/definitions/ip","default":"127.0.0.1"},"port":{"description":"Port of the neovim plugin(neopyter) is located, neovim will listen to the port. e.g. `9001`","$ref":"#/definitions/port","default":9001}}},{"properties":{"mode":{"description":"In this mode, jupyter lab server(server side, the host you run `jupyter lab` to start JupyterLab) is server and jupyter lab server extension(neopyter) is listening to `${IP}:{Port}`, the neovim plugin(neopyter) will connect to `${IP}:{Port}`","enum":["proxy"]},"ip":{"description":"IP of the host where jupyter server is located, jupyter lab server extension(neopyter) will listen to the IP, please use `,` to split multiple. e.g. `127.0.0.1,10.25.69.51`","$ref":"#/definitions/ip","default":"127.0.0.1"},"port":{"description":"Port of the jupyter server extension(neopyter) is listening to. e.g. `9001`","$ref":"#/definitions/port","default":9001}}}]}},"type":"object"}')}}]);