"use strict";(self.webpackChunkneopyter=self.webpackChunkneopyter||[]).push([[330],{8330:e=>{e.exports=JSON.parse('{"name":"neopyter","version":"0.2.6","description":"A JupyterLab extension. Integrate JupyterLab and Neovim","keywords":["jupyter","jupyterlab","jupyterlab-extension","neovim"],"homepage":"https://github.com/SUSTech-data/neopyter","bugs":{"url":"https://github.com/SUSTech-data/neopyter/issues"},"license":"BSD-3-Clause","author":{"name":"Abao Zhang","email":"abaodoit@gmail.com"},"files":["lib/**/*.{d.ts,eot,gif,html,jpg,js,js.map,json,png,svg,woff2,ttf}","style/**/*.{css,js,eot,gif,html,jpg,json,png,svg,woff2,ttf}","schema/*.json"],"main":"lib/index.js","types":"lib/index.d.ts","style":"style/index.css","repository":{"type":"git","url":"https://github.com/SUSTech-data/neopyter"},"scripts":{"build":"pnpm build:lib && pnpm build:labextension:dev","build:prod":"run-s clean build:lib:prod build:labextension","build:labextension":"jupyter labextension build .","build:labextension:dev":"jupyter labextension build --development True .","build:lib":"tsc","build:lib:prod":"tsc","clean":"pnpm clean:lib","clean:lib":"rimraf lib tsconfig.tsbuildinfo","clean:lintcache":"rimraf .eslintcache .stylelintcache","clean:labextension":"rimraf neopyter/labextension neopyter/_version.py","clean:all":"pnpm clean:lib && pnpm clean:labextension && pnpm clean:lintcache","eslint":"pnpm eslint:check --fix","eslint:check":"eslint . --cache --ext .ts,.tsx","install:extension":"pnpm build","lint":"pnpm stylelint && pnpm prettier && pnpm eslint","lint:check":"pnpm stylelint:check && pnpm prettier:check && pnpm eslint:check","prettier":"pnpm prettier:base --write --list-different","prettier:base":"prettier \\"**/*{.ts,.tsx,.js,.jsx,.css}\\"","prettier:check":"pnpm prettier:base --check","stylelint":"pnpm stylelint:check --fix","stylelint:check":"stylelint --cache \\"style/**/*.css\\"","test":"jest --coverage","watch":"run-p watch:src watch:labextension","watch:src":"tsc -w --strict false --sourceMap","watch:labextension":"jupyter labextension watch .","prepare":"husky install"},"dependencies":{"@jupyterlab/application":"^4.1.5","@jupyterlab/apputils":"^4.2.5","@jupyterlab/logconsole":"^4.1.5","@jupyterlab/notebook":"^4.1.5","@lumino/widgets":"^2.3.1","@msgpack/msgpack":"3.0.0-beta2","@rjsf/antd":"^5.18.2","@rjsf/core":"^5.18.2","@rjsf/utils":"^5.18.2","@rjsf/validator-ajv8":"^5.18.2","@types/react":"^18.3.3","antd":"^5.16.2","react":"^18.3.1","remeda":"^1.55.0","zustand":"^4.5.2"},"devDependencies":{"@commitlint/cli":"^18.6.1","@commitlint/config-conventional":"^18.6.3","@jupyterlab/builder":"^4.1.5","@jupyterlab/completer":"^4.1.5","@jupyterlab/coreutils":"^6.1.5","@jupyterlab/docmanager":"^4.1.5","@jupyterlab/services":"^7.1.5","@jupyterlab/settingregistry":"^4.1.5","@jupyterlab/testutils":"^4.1.5","@jupyterlab/ui-components":"^4.1.5","@lumino/coreutils":"^2.1.2","@lumino/disposable":"^2.1.2","@types/jest":"^29.5.12","@types/json-schema":"^7.0.15","@typescript-eslint/eslint-plugin":"^6.21.0","@typescript-eslint/parser":"^6.21.0","css-loader":"^6.10.0","eslint":"^8.57.0","eslint-config-prettier":"^8.10.0","eslint-plugin-prettier":"^5.1.3","husky":"^8.0.3","jest":"^29.7.0","npm-run-all":"^4.1.5","prettier":"^3.2.5","rimraf":"^5.0.5","source-map-loader":"^1.1.3","style-loader":"^3.3.4","stylelint":"^15.11.0","stylelint-config-recommended":"^13.0.0","stylelint-config-standard":"^34.0.0","stylelint-prettier":"^4.1.0","typescript":"~5.0.4","yjs":"^13.6.14"},"sideEffects":["style/*.css","style/index.js"],"styleModule":"style/index.js","publishConfig":{"access":"public"},"jupyterlab":{"extension":true,"schemaDir":"schema","outputDir":"neopyter/labextension"},"prettier":{"singleQuote":true,"trailingComma":"none","arrowParens":"avoid","endOfLine":"auto","overrides":[{"files":"package.json","options":{"tabWidth":4}}]},"stylelint":{"extends":["stylelint-config-recommended","stylelint-config-standard","stylelint-prettier/recommended"],"rules":{"property-no-vendor-prefix":null,"selector-no-vendor-prefix":null,"value-no-vendor-prefix":null}}}')}}]);